@@ -1,5 +1,5 @@
 #!/bin/bash
-#  Copyright (C) 2000-2007 SWsoft. All rights reserved.
+#  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -16,22 +16,18 @@
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 #
-# This script deletes IP address inside VE for Slackware like systems.
-#
-# Parameters are passed in environment variables.
-# Required parameters:
-#   IP_ADDR       - IPs to delete, several addresses should be divided by space
-# Optional parameters:
-#   IPDELALL      - delete all ip addresses
+# Deletes IP address from a container running Slackware.
+
 IFCFG=/etc/rc.d/rc.inet1.conf
 
 function del_ip()
 {
-	local ip
+	local ipm
 
 	[ -f ${IFCFG}  ] || return 0
-	for ip in ${IP_ADDR}; do
-		if grep -wq "${ip}" ${IFCFG} 2>/dev/null; then
+	for ipm in ${IP_ADDR}; do
+		ip_conv $ipm
+		if grep -wq "${_IP}" ${IFCFG} 2>/dev/null; then
 			/etc/rc.d/rc.inet1 stop
 			break
 		fi
