@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2010, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2011, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -48,6 +48,7 @@
 #include "meminfo.h"
 #include "vzfeatures.h"
 #include "io.h"
+#include "net.h"
 
 static int _page_size;
 static int check_name(char *name);
@@ -141,8 +142,10 @@ static vps_config config[] = {
 {"IOPRIO",	NULL, PARAM_IOPRIO},
 {"BOOTORDER",	NULL, PARAM_BOOTORDER},
 
-/* These ones are known parameters for global config file,
- * it's just vzctl is not interested in those
+/* These ones are either known parameters for global config file,
+ * or some obsoleted parameters used in the past. In both cases
+ * vzctl is not interested in those, but it doesn't make sense to
+ * report those as being unknown.
  */
 {"VIRTUOZZO",		NULL, PARAM_IGNORED},
 {"VE0CPUUNITS",		NULL, PARAM_IGNORED},
@@ -152,6 +155,7 @@ static vps_config config[] = {
 {"VZWDOG",		NULL, PARAM_IGNORED},
 {"IPTABLES_MODULES",	NULL, PARAM_IGNORED},
 {"IP6TABLES",		NULL, PARAM_IGNORED},
+{"CONFIG_CUSTOMIZED",	NULL, PARAM_IGNORED},
 
 {NULL,		NULL, -1}
 };
@@ -485,12 +489,12 @@ static int store_env(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 }
 
 /********************** UB **************************************/
-static long get_mul(char c)
+static long long get_mul(char c)
 {
 	switch (c) {
 	case 'T':
 	case 't':
-		return 1024l * 1024 * 1024 * 1024;
+		return 1024ll * 1024 * 1024 * 1024;
 	case 'G':
 	case 'g':
 		return 1024 * 1024 * 1024;
@@ -514,7 +518,7 @@ static long get_mul(char c)
  */
 static const char *parse_ul_sfx(const char *str, unsigned long long *val, int divisor)
 {
-	long n;
+	long long n;
 	char *tail;
 	long double v = 0;
 
@@ -659,8 +663,8 @@ static int store_ub(vps_param *old_p, vps_param *vps_p,
 #define ADD_UB_PARAM(res, id)						\
 if (ub->res != NULL) {							\
 	conf = conf_get_by_id(config, id);				\
-	snprintf(buf, sizeof(buf), "%s=\"%lu:%lu\"", conf->name,	\
-		ub->res[0], ub->res[1]);				\
+	snprintf(buf, sizeof(buf), "%s=\"%s\"", conf->name,		\
+		ubcstr(ub->res[0], ub->res[1]));			\
 	if (add_str_param(conf_h, buf))					\
 		return ERR_NOMEM;					\
 }
@@ -702,20 +706,14 @@ static int parse_cap(char *str, cap_param *cap)
 	unsigned long *mask;
 
 	for_each_strtok(token, str, "\t ") {
-		if ((p = strrchr(token, ':')) == NULL) {
-			logger(-1, 0, "Invalid syntaxes in %s:"
-				" capname:on|off", token);
-			return ERR_INVAL;
-		}
+		if ((p = strrchr(token, ':')) == NULL)
+			goto err;
 		if (!strcmp(p + 1, "off"))
 			mask = &cap->off;
 		else if (!strcmp(p + 1, "on"))
 			mask = &cap->on;
-		else {
-			logger(-1, 0, "Invalid syntaxes in %s:"
-				" capname:on|off", token);
-			return ERR_INVAL;
-		}
+		else
+			goto err;
 		len = p - token;
 		strncpy(cap_nm, token,
 			len < sizeof(cap_nm) ? len : sizeof(cap_nm));
@@ -727,6 +725,10 @@ static int parse_cap(char *str, cap_param *cap)
 	}
 
 	return 0;
+err:
+	logger(-1, 0, "Can't parse capability %s (expecting capname:on|off)",
+			token);
+	return ERR_INVAL;
 }
 
 static int store_cap(vps_param *old_p, vps_param *vps_p, vps_config *conf,
@@ -751,28 +753,10 @@ static int store_cap(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 }
 /********************** Network ************************/
 
-static int check_ip_dot(char *ip)
-{
-	int i;
-	char *str = ip;
-	char *p;
-
-	for (i = 0; i < 5; i++) {
-		if ((p = strchr(str, '.')) == NULL)
-			break;
-		str = p + 1;
-	}
-	if (i != 3)
-		return VZ_BADIP;
-	return 0;
-}
-
 static int parse_ip(vps_param *vps_p, char *val, int id)
 {
 	char *token;
-	char dst[INET6_ADDRSTRLEN];
-	unsigned char ip[sizeof(struct in6_addr)];
-	int family;
+	char *dst;
 	net_param *net;
 
 	if (id == PARAM_IP_ADD)
@@ -794,13 +778,8 @@ static int parse_ip(vps_param *vps_p, char *val, int id)
 			continue;
 		if (!strcmp(token, "::0"))
 			continue;
-		if (!strchr(token, ':')) {
-			if (check_ip_dot(token))
-				return ERR_INVAL;
-		}
-		if ((family = get_netaddr(token, ip)) < 0)
-			return ERR_INVAL;
-		if (inet_ntop(family, ip, dst, sizeof(dst)) == NULL)
+		dst = canon_ip(token);
+		if (dst == NULL)
 			return ERR_INVAL;
 		if (!find_ip(&net->ip, dst))
 			add_str_param(&net->ip, dst);
@@ -815,7 +794,6 @@ static int store_ip(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	list_head_t ip;
 	int ret;
 
-	list_head_init(&ip);
 	if (conf->id != PARAM_IP_ADD)
 		return 0;
 	if (!vps_p->res.net.delall &&
@@ -824,7 +802,8 @@ static int store_ip(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	{
 		return 0;
 	}
-	merge_str_list(vps_p->res.net.delall, &old_p->res.net.ip,
+	list_head_init(&ip);
+	merge_ip_list(vps_p->res.net.delall, &old_p->res.net.ip,
 		&vps_p->res.net.ip, &vps_p->del_res.net.ip, &ip);
 	ret = conf_store_strlist(conf_h, conf->name, &ip, 1);
 	free_str_param(&ip);
@@ -866,7 +845,6 @@ static int store_netdev(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	list_head_t dev;
 	int ret;
 
-	list_head_init(&dev);
 	if (conf->id != PARAM_NETDEV_ADD)
 		return 0;
 	if (list_empty(&vps_p->res.net.dev) &&
@@ -874,6 +852,7 @@ static int store_netdev(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	{
 		return 0;
 	}
+	list_head_init(&dev);
 	merge_str_list(0, &old_p->res.net.dev, &vps_p->res.net.dev,
 		 &vps_p->del_res.net.dev, &dev);
 	ret = conf_store_strlist(conf_h, conf->name, &dev, 1);
@@ -996,35 +975,62 @@ static int parse_dev_perm(const char *str, unsigned int *perms)
 	return 0;
 }
 
+static const char* devperm2str(unsigned long perms)
+{
+	static char mask[4];
+	int i=0;
+	if (perms & S_IROTH)
+		mask[i++] = 'r';
+	if (perms & S_IWOTH)
+		mask[i++] = 'w';
+	if (perms & S_IXGRP)
+		mask[i++] = 'q';
+	mask[i] = 0;
+	return mask;
+}
+
+/* Do not allow devices with specific majors */
+static int dev_major_denied(int major)
+{
+	/* loop device */
+	if (major == 7)
+		return 1;
+
+	return 0;
+}
+
 static int parse_devices_str(const char *str, dev_res *dev)
 {
-	int ret;
-	unsigned long val, major;
-	char type[2];
-	char minor[32];
-	char mode[3];
+	int minor, major;
+	char type;
+	char minor_str[16];
+	char mode[6];
 
-	ret = sscanf(str, "%1[^:]:%lu:%16[^:]:%2s", type, &major, minor, mode);
-	if (ret != 4)
-		return 1;
+	if (sscanf(str, "%c:%d:%15[^:]:%5s",
+			&type, &major, minor_str, mode) != 4)
+		return -1;
+	if (dev_major_denied(major))
+		return -1;
 	memset(dev, 0, sizeof(*dev));
-	if (!strcmp(type, "b"))
+	if (type == 'b')
 		dev->type = S_IFBLK;
-	else if (!strcmp(type, "c"))
+	else if (type == 'c')
 		dev->type = S_IFCHR;
 	else
 		return -1;
-	if (!strcmp(minor, "all")) {
+	if (!strcmp(minor_str, "all")) {
 		dev->use_major = VE_USE_MAJOR;
 		dev->type |= VE_USE_MAJOR;
 		dev->dev = makedev(major, 0);
 	} else {
 		dev->type |= VE_USE_MINOR;
-		if (parse_ul(minor, &val))
+		if (parse_int(minor_str, &minor))
 			return -1;
-		dev->dev = makedev(major, val);
+		dev->dev = makedev(major, minor);
 	}
-	ret = parse_dev_perm(mode, &dev->mask);
+	if (parse_dev_perm(mode, &dev->mask) != 0)
+		return -1;
+
 	return 0;
 }
 
@@ -1046,20 +1052,6 @@ static int parse_dev(vps_param *vps_p, char *val)
 	return 0;
 }
 
-static const char* devperm2str(unsigned long perms)
-{
-	static char mask[4];
-	int i=0;
-	if (perms & S_IROTH)
-		mask[i++] = 'r';
-	if (perms & S_IWOTH)
-		mask[i++] = 'w';
-	if (perms & S_IXGRP)
-		mask[i++] = 'q';
-	mask[i] = 0;
-	return mask;
-}
-
 static int store_dev(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	list_head_t *conf_h)
 {
@@ -1144,6 +1136,12 @@ static int parse_devnodes_str(const char *str, dev_res *dev)
 		logger(-1, 0, "The %s is not block or character device", buf);
 		goto err;
 	}
+	if (dev_major_denied(major(st.st_rdev))) {
+		logger(-1, 0, "Device %s is not allowed "
+				"to be assigned to a CT",
+				dev->name);
+		goto err;
+	}
 	dev->dev = st.st_rdev;
 	dev->type |= VE_USE_MINOR;
 	if (parse_dev_perm(ch, &dev->mask))
