@@ -115,8 +115,8 @@ err:
 	return ret ? err : 0;
 }
 
-int real_chkpnt(int cpt_fd, envid_t veid, const char *root, cpt_param *param,
-	int cmd)
+static int real_chkpnt(int cpt_fd, envid_t veid, const char *root,
+		cpt_param *param, int cmd)
 {
 	int ret, len, len1;
 	char buf[PIPE_BUF];
@@ -244,7 +244,7 @@ int vps_chkpnt(vps_handler *h, envid_t veid, vps_param *vps_p, int cmd,
 		}
 	} else {
 		if (ioctl(cpt_fd, CPT_SET_VEID, veid) < 0) {
-			logger(0, errno, "Can not set CT ID");
+			logger(-1, errno, "Can not set CT ID");
 			goto err;
 		}
 	}
@@ -455,7 +455,8 @@ int vps_restore(vps_handler *h, envid_t veid, vps_param *vps_p, int cmd,
 		goto err;
 	/* Restore second-level quota links & quota device */
 	if ((cmd == CMD_RESTORE || cmd == CMD_UNDUMP) &&
-		vps_p->res.dq.ugidlimit != NULL && vps_p->res.dq.ugidlimit)
+		vps_p->res.dq.ugidlimit != NULL &&
+		*vps_p->res.dq.ugidlimit != 0)
 	{
 		logger(0, 0, "Restore second-level quota");
 		if (vps_execFn(h, veid, vps_p->res.fs.root, mk_quota_link, NULL,
