@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2011, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -43,17 +43,44 @@
 #include "vps_configure.h"
 
 
-int find_ip(list_head_t *ip_h, char *ipaddr)
+char *find_ip(list_head_t *ip_h, const char *ipaddr)
 {
 	ip_param *ip;
+	char *slash, *ip_slash, *ip_only;
+	int len;
 
 	if (list_empty(ip_h))
-		return 0;
-	list_for_each(ip, ip_h, list) {
-		if (!strcmp(ip->val, ipaddr))
-			return 1;
+		return NULL;
+
+	slash = strchr(ipaddr, '/');
+	if (slash) {
+		len = slash - ipaddr + 1;
+		ip_slash = strndupa(ipaddr, len);
+		ip_only = strndupa(ipaddr, len - 1);
 	}
-	return 0;
+	else {
+		ip_only = NULL;
+		len = asprintf(&ip_slash, "%s/", ipaddr);
+	}
+
+	list_for_each(ip, ip_h, list) {
+		/* Match complete IP or IP/mask string */
+		if (!strcmp(ip->val, ipaddr))
+			return ip->val;
+		/* Match only IP */
+		if ((ip_only != NULL) && (!strcmp(ip->val, ip_only)))
+			return ip->val;
+		/* Match IP/ (same IP, different mask) */
+		if (!strncmp(ip->val, ip_slash, len))
+			return ip->val;
+	}
+	return NULL;
+}
+
+int merge_ip_list(int delall, list_head_t *old, list_head_t *add,
+		list_head_t *del, list_head_t *merged)
+{
+	return __merge_str_list(delall, old, add, del, merged, find_ip);
 }
 
 static inline int _ip_ctl(vps_handler *h, envid_t veid, int op,
@@ -86,7 +113,7 @@ static inline int _ip_ctl(vps_handler *h, envid_t veid, int op,
 	return ioctl(h->vzfd, VENETCTL_VE_IP_MAP, &ip_map);
 }
 
-int ip_ctl(vps_handler *h, envid_t veid, int op, char *ip)
+static int ip_ctl(vps_handler *h, envid_t veid, int op, char *ip)
 {
 	int ret;
 	int family;
@@ -188,7 +215,7 @@ static int vps_ip_ctl(vps_handler *h, envid_t veid, int op,
 	return ret;
 }
 
-int vps_add_ip(vps_handler *h, envid_t veid,
+static int vps_add_ip(vps_handler *h, envid_t veid,
 	net_param *net, int state)
 {
 	char *str;
@@ -208,7 +235,7 @@ int vps_add_ip(vps_handler *h, envid_t veid,
 	return ret;
 }
 
-int vps_del_ip(vps_handler *h, envid_t veid,
+static int vps_del_ip(vps_handler *h, envid_t veid,
 	net_param *net, int state)
 {
 	char *str;
@@ -227,7 +254,7 @@ int vps_del_ip(vps_handler *h, envid_t veid,
 	return ret;
 }
 
-int vps_set_ip(vps_handler *h, envid_t veid,
+static int vps_set_ip(vps_handler *h, envid_t veid,
 	net_param *net, int state)
 {
 	int ret;
@@ -258,7 +285,7 @@ static int netdev_ctl(vps_handler *h, int veid, int op, char *name)
 	return 0;
 }
 
-int set_netdev(vps_handler *h, envid_t veid, int cmd, net_param *net)
+static int set_netdev(vps_handler *h, envid_t veid, int cmd, net_param *net)
 {
 	int ret = 0;
 	list_head_t *dev_h = &net->dev;
@@ -328,7 +355,7 @@ static int remove_ipv6_addr(net_param *net)
 
 	cnt = 0;
 	list_for_each_safe(ip, tmp, head, list) {
-		if (strchr(ip->val, ':')) {
+		if (get_addr_family(ip->val) == AF_INET6) {
 			free(ip->val);
 			list_del(&ip->list);
 			free(ip);
@@ -369,7 +396,7 @@ int vps_net_ctl(vps_handler *h, envid_t veid, int op, net_param *net,
 }
 
 #define	PROC_VEINFO	"/proc/vz/veinfo"
-int get_vps_ip_proc(envid_t veid, list_head_t *ip_h)
+static int get_vps_ip_proc(envid_t veid, list_head_t *ip_h)
 {
 	FILE *fd;
 	char str[16384];
@@ -396,10 +423,12 @@ int get_vps_ip_proc(envid_t veid, list_head_t *ip_h)
 		if (token == NULL)
 			break;
 		while ((token = strtok(NULL, " \t\n")) != NULL) {
-			if (strchr(token, ':') &&
+			/* Canonicalize IPv6 addresses */
+			if ((get_addr_family(token) == AF_INET6) &&
 			    inet_pton(AF_INET6, token, data) > 0 &&
 			    !inet_ntop(AF_INET6, data, token, strlen(token)+1))
 				break;
+
 			if (add_str_param(ip_h, token)) {
 				free_str_param(ip_h);
 				cnt = -1;
@@ -422,7 +451,7 @@ int get_vps_ip_proc(envid_t veid, list_head_t *ip_h)
 	((unsigned char *)&addr)[3]
 #endif
 
-int get_vps_ip_ioctl(vps_handler *h, envid_t veid,
+static int get_vps_ip_ioctl(vps_handler *h, envid_t veid,
 	list_head_t *ip_h)
 {
 	int ret = -1;
